@@ -16,15 +16,24 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#include <map>
+
+#include <glog/logging.h>
+
 #include <QBuffer>
 #include <QCryptographicHash>
+#include <QDBusConnection>
+#include <QDBusMessage>
 #include <QDir>
-#include <QStringList>
 #include <QFile>
 #include <QFileInfo>
+#include <QMimeDatabase>
+#include <QMimeType>
+#include <QStringList>
 #include <QSslError>
-#include <glog/logging.h>
-#include <map>
+
+#include <ubuntu/transfers/metadata.h>
+#include <ubuntu/transfers/system/dbus_connection.h>
 #include <ubuntu/transfers/system/hash_algorithm.h>
 #include <ubuntu/transfers/system/cryptographic_hash.h>
 #include <ubuntu/transfers/system/logger.h>
@@ -32,11 +41,12 @@
 #include <ubuntu/transfers/system/filename_mutex.h>
 #include <ubuntu/transfers/system/uuid_factory.h>
 #include <ubuntu/transfers/system/uuid_utils.h>
+
+#include "header_parser.h"
 #include "file_download.h"
 
 #define DOWN_LOG(LEVEL) LOG(LEVEL) << ((parent() != nullptr)?"GroupDownload {" + parent()->objectName() + " } ":"") << "Download ID{" << objectName() << " } "
 
-
 std::ostream& operator<<(std::ostream &out, QNetworkReply::NetworkError err) {
     static std::map<QNetworkReply::NetworkError, std::string> errorsMap {
         {QNetworkReply::ConnectionRefusedError, "ConnectionRefusedError: the remote "
@@ -131,7 +141,12 @@ std::ostream& operator<<(std::ostream &out, QNetworkReply::NetworkError err) {
 }
 
 namespace {
-
+    const QString PROPERTIES_INTERFACE = "org.freedesktop.DBus.Properties";
+    const QString DOWNLOAD_INTERFACE = "com.canonical.applications.Download";
+    const QString PROPERTIES_CHANGED = "PropertiesChanged";
+    const QString CLICK_PACKAGE_PROPERTY = "ClickPackage";
+    const QString SHOW_INDICATOR_PROPERTY = "ShowInIndicator";
+    const QString TITLE_PROPERTY = "Title";
     const QString DATA_FILE_NAME = "data.download";
     const QString NETWORK_ERROR = "NETWORK ERROR";
     const QString HASH_ERROR = "HASH ERROR";
@@ -143,6 +158,8 @@ namespace {
     const QString PROXY_AUTH_ERROR = "PROXY_AUTHENTICATION ERROR";
     const QString UNEXPECTED_ERROR = "UNEXPECTED_ERROR";
     const QByteArray CONTENT_DISPOSITION = "Content-Disposition";
+    const QByteArray CONTENT_TYPE = "Content-Type";
+    const QString DATA_URI_PREFIX = "data:";
 }
 
 namespace Ubuntu {
@@ -227,31 +244,37 @@ FileDownload::cancelTransfer() {
 void
 FileDownload::pauseTransfer() {
     TRACE << _url;
-    if (_reply == nullptr) {
-        // cannot pause because is not running
-        DOWN_LOG(INFO) << "Cannot pause download because reply is NULL";
-        DOWN_LOG(INFO) << "EMIT paused(false)";
-        emit paused(false);
-        return;
-    }
-
-    DOWN_LOG(INFO) << "Pausing download" << _url;
-    // we need to disconnect the signals to ensure that they are not
-    // emitted due to the operation we are going to perform. We read
-    // the data in the reply and store it in a file
-    disconnectFromReplySignals();
-
-    // do abort before reading
-    _reply->abort();
-    _currentData->write(_reply->readAll());
-    if (!flushFile()) {
+    if (_url.toString().contains(DATA_URI_PREFIX)) {
+        DOWN_LOG(INFO) << "EMIT paused(false) since we are using a data uri";
+        _downloading = false;
         emit paused(false);
     } else {
-        _reply->deleteLater();
-        _reply = nullptr;
-        DOWN_LOG(INFO) << "EMIT paused(true)";
-        _downloading = false;
-        emit paused(true);
+        if (_reply == nullptr) {
+            // cannot pause because is not running
+            DOWN_LOG(INFO) << "Cannot pause download because reply is NULL";
+            DOWN_LOG(INFO) << "EMIT paused(false)";
+            emit paused(false);
+            return;
+        }
+
+        DOWN_LOG(INFO) << "Pausing download" << _url;
+        // we need to disconnect the signals to ensure that they are not
+        // emitted due to the operation we are going to perform. We read
+        // the data in the reply and store it in a file
+        disconnectFromReplySignals();
+
+        // do abort before reading
+        _reply->abort();
+        _currentData->write(_reply->readAll());
+        if (!flushFile()) {
+            emit paused(false);
+        } else {
+            _reply->deleteLater();
+            _reply = nullptr;
+            DOWN_LOG(INFO) << "EMIT paused(true)";
+            _downloading = false;
+            emit paused(true);
+        }
     }
 }
 
@@ -267,23 +290,32 @@ FileDownload::resumeTransfer() {
         return;
     }
 
-    DOWN_LOG(INFO) << "Resuming download.";
-    QNetworkRequest request = buildRequest();
+    // it is not very probable, yet possible that we do reach this point with a data uri
 
-    // overrides the range header, we do not let clients set the range!!!
-    qint64 currentDataSize = _currentData->size();
-    QByteArray rangeHeaderValue = "bytes=" +
-        QByteArray::number(currentDataSize) + "-";
-    request.setRawHeader("Range", rangeHeaderValue);
+    if (_url.toString().contains(DATA_URI_PREFIX)) {
+        DOWN_LOG(INFO) << "EMIT resumed(true)";
+        _downloading = true;
+        emit resumed(true);
+        writeDataUri();
+    } else {
+        DOWN_LOG(INFO) << "Resuming download.";
+        QNetworkRequest request = buildRequest();
 
-    _reply = _requestFactory->get(request);
-    _reply->setReadBufferSize(throttle());
+        // overrides the range header, we do not let clients set the range!!!
+        qint64 currentDataSize = _currentData->size();
+        QByteArray rangeHeaderValue = "bytes=" +
+                QByteArray::number(currentDataSize) + "-";
+        request.setRawHeader("Range", rangeHeaderValue);
 
-    connectToReplySignals();
+        _reply = _requestFactory->get(request);
+        _reply->setReadBufferSize(throttle());
 
-    DOWN_LOG(INFO) << "EMIT resumed(true)";
-    _downloading = true;
-    emit resumed(true);
+        connectToReplySignals();
+
+        DOWN_LOG(INFO) << "EMIT resumed(true)";
+        _downloading = true;
+        emit resumed(true);
+    }
 }
 
 void
@@ -308,16 +340,29 @@ FileDownload::startTransfer() {
         return;
     }
 
-    DOWN_LOG(INFO) << "Network is accessible, performing download request";
-    // signals should take care of calling deleteLater on the
-    // NetworkReply object
-    _reply = _requestFactory->get(buildRequest());
-    _reply->setReadBufferSize(throttle());
+    // make a diff between a data uri and a "normal" download
+    if (_url.toString().contains(DATA_URI_PREFIX)) {
+        DOWN_LOG(INFO) << "Performing a data uri download.";
+        // we emit the signals and then write the data in the file system, which will emit the download finished signal
+        // the signals are queued so they will happen in the correct order
 
-    connectToReplySignals();
-    DOWN_LOG(INFO) << "EMIT started(true)";
-    _downloading = true;
-    emit started(true);
+        DOWN_LOG(INFO) << "EMIT started(true)";
+        _downloading = true;
+        emit started(true);
+
+        writeDataUri();
+    } else {
+        DOWN_LOG(INFO) << "Performing a network download.";
+        // signals should take care of calling deleteLater on the
+        // NetworkReply object
+        _reply = _requestFactory->get(buildRequest());
+        _reply->setReadBufferSize(throttle());
+
+        connectToReplySignals();
+        DOWN_LOG(INFO) << "EMIT started(true)";
+        _downloading = true;
+        emit started(true);
+    }
 }
 
 qulonglong
@@ -413,6 +458,37 @@ FileDownload::setHeaders(StringMap headers) {
     }
 }
 
+void
+FileDownload::setMetadata(const QVariantMap& data) {
+    TRACE << data;
+
+    // we want to emit the changed signals for those keys that represent
+    // properties
+    QVariantMap changes;
+    if (data.contains(Metadata::TITLE_KEY)) {
+        if (data[Metadata::TITLE_KEY] != _metadata[Metadata::TITLE_KEY]) {
+            changes[TITLE_PROPERTY] = data[Metadata::TITLE_KEY];
+        }
+    }
+
+    if (data.contains(Metadata::SHOW_IN_INDICATOR_KEY)) {
+        if (data[Metadata::SHOW_IN_INDICATOR_KEY]
+                != _metadata[Metadata::SHOW_IN_INDICATOR_KEY]) {
+            changes[SHOW_INDICATOR_PROPERTY] = data[Metadata::SHOW_IN_INDICATOR_KEY];
+        }
+    }
+
+    if (data.contains(Metadata::CLICK_PACKAGE_KEY)) {
+        if (data[Metadata::CLICK_PACKAGE_KEY]
+                != _metadata[Metadata::CLICK_PACKAGE_KEY]) {
+            changes[CLICK_PACKAGE_PROPERTY] = data[Metadata::CLICK_PACKAGE_KEY];
+        }
+    }
+
+    Download::setMetadata(data);
+    emit propertiesChanged(changes);
+}
+
 void
 FileDownload::onDownloadProgress(qint64 currentProgress, qint64 bytesTotal) {
     TRACE << _url << currentProgress << bytesTotal;
@@ -423,7 +499,7 @@ FileDownload::onDownloadProgress(qint64 currentProgress, qint64 bytesTotal) {
     if (!flushFile()) {
         return;
     }
-    qulonglong received = _currentData->size();
+    auto received = static_cast<qulonglong>(_currentData->size());
 
     if (bytesTotal == -1) {
         // we do not know the size of the download, simply return
@@ -432,12 +508,11 @@ FileDownload::onDownloadProgress(qint64 currentProgress, qint64 bytesTotal) {
         return;
     } else {
         if (_totalSize == 0) {
-            qlonglong uBytestTotal = bytesTotal;
             // bytesTotal is different when we have resumed because we
             // are not counting the size that  we already downloaded,
             // therefore we only do this once
             // update the metadata
-            _totalSize = uBytestTotal;
+            _totalSize = static_cast<qulonglong>(bytesTotal);
         }
         emit Download::progress(received, _totalSize);
         return;
@@ -527,121 +602,127 @@ FileDownload::onRedirect(QUrl redirect) {
 void
 FileDownload::onDownloadCompleted() {
     TRACE << _url;
+    // ensure that if content-disposition is present we will use it
+    updateFileNamePerContentDisposition();
 
-    // check if we have the content-type header, if we do we are going to change the
-    // file path that will be used by the download, do not do it if the app is
-    // unconfined
-    if ((_reply->hasRawHeader(CONTENT_DISPOSITION) && (
-            isConfined() || !metadata().contains(Metadata::LOCAL_PATH_KEY)))) {
-        QString contentDisposition = _reply->rawHeader(CONTENT_DISPOSITION);
-        DOWN_LOG(INFO) << "Content-Disposition header" << contentDisposition;
+    auto contentType = (_reply->hasRawHeader(CONTENT_TYPE))?
+            QString(_reply->rawHeader(CONTENT_TYPE)) : QString();
 
-        if (contentDisposition.contains("filename")) {
-            auto serverName = filenameFromHTTPContentDisposition(contentDisposition);
-            DOWN_LOG(INFO) << "Server name " << serverName;
-
-            if (!serverName.isEmpty()) {
-                QFileInfo fiContentDisposition(serverName);
-                auto filename = fiContentDisposition.fileName();
-                // replace the filename of the current _filePath with the new one
-                QFileInfo fiFilePath(_filePath);
-                auto currentFileName = fiFilePath.fileName();
-                auto newPath = _filePath.replace(currentFileName, filename);
-
-                // unlock the old path and lock the new one
-                _fileNameMutex->unlockFileName(_filePath);
-                _filePath = _fileNameMutex->lockFileName(newPath);
-                DOWN_LOG(INFO) << "Content disposition based file path is '" << serverName << "'";
-            }
-        }
-    }
-
-    // if the hash is present we check it
-    if (!_hash.isEmpty()) {
-        emit processing(filePath());
-        _currentData->reset();
-        auto hashFactory = CryptographicHashFactory::instance();
-        QScopedPointer<CryptographicHash> hash(
-            hashFactory->createCryptographicHash(_algo, this));
-        // addData is smart enough to not load the entire file in memory
-        hash->addData(_currentData->device());
-        QString fileSig = QString(hash->result().toHex());
-
-        if (fileSig != _hash) {
-            DOWN_LOG(ERROR) << HASH_ERROR << fileSig << "!=" << _hash;
-            emitError(HASH_ERROR);
-            return;
-        }
-    }
-
-    // there are two possible cases, the first, we do not have the metadata
-    // info to execute a command once the download was finished and that
-    // means we are done here else we execute the command AND raise the
-    // finish signals once the command was done (or an error occurred in
-    // the command execution.
-    if (metadata().contains(Metadata::COMMAND_KEY)) {
-        // just emit processing if we DO NOT have a hash because else we
-        // already emitted it.
-        if (_hash.isEmpty()) {
-            emit processing(filePath());
-        }
-        // toStringList will return an empty list if it cannot be converted
-        QStringList commandData =
-            metadata()[Metadata::COMMAND_KEY].toStringList();
-        if (commandData.count() == 0) {
-            DOWN_LOG(ERROR) << "COMMAND DATA MISSING";
-            emitError(COMMAND_ERROR);
-            return;
-        } else {
-            // first item of the string list is the command
-            // the rest is the arguments
-            DOWN_LOG(INFO) << "Renaming '" << _tempFilePath << "'"
-                << "' to '" << _filePath << "'";
-
-            auto fileMan = FileManager::instance();
-            if (fileMan->exists(_tempFilePath)) {
-                LOG(INFO) << "Renaming: '" << _tempFilePath << "' to '"
-                    << _filePath << "'";
-                fileMan->rename(_tempFilePath, _filePath);
-            }
-
-            QString command = commandData.at(0);
-            commandData.removeAt(0);
-            QStringList args;
-
-            foreach(const QString& arg, commandData) {
-                if (arg == Metadata::COMMAND_FILE_KEY)
-                    args << filePath();
-                else
-                    args << arg;
-            }
-
-            Process* postDownloadProcess =
-                ProcessFactory::instance()->createProcess();
-
-            // connect to signals so that we can tell the clients that
-            // the operation succeed
-
-            CHECK(connect(postDownloadProcess, &Process::finished,
-                this, &FileDownload::onProcessFinished))
-                    << "Could not connect to signal";
-            CHECK(connect(postDownloadProcess, &Process::error,
-                this, &FileDownload::onProcessError))
-                    << "Could not connect to signal";
-
-            DOWN_LOG(INFO) << "Executing" << command << args;
-            postDownloadProcess->start(command, args);
-            return;
-        }
-    } else {
-        emitFinished();
-    }
+    downloadPostProcessing(contentType);
 
     // clean the reply
     _reply->deleteLater();
     _reply = nullptr;
 }
 
+void
+FileDownload::writeDataUri() {
+    // the expected format of the data is as follows:
+    //
+    // data:[<MIME-type>][;charset=<encoding>][;base64],<data>
+    //
+    // The encoding is indicated by ;base64. If it is present the data is encoded as base64. Without it the data
+    // (as a sequence of octets) is represented using ASCII encoding for octets inside the range of safe URL
+    // characters.  If <MIME-type> is omitted, it defaults to text/plain;charset=US-ASCII.
+    // An example uri is:
+    //
+    // data:image/gif;base64,R0lGODlhAQABAIAAAP///////yH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==
+
+    // HACK:
+    // it can also be seen with the url prefixed as it happens in google images:
+    //
+    // http://images.google.com/data:image/gif;base64,R0lGODlhAQABAIAAAP///////yH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==
+    //
+    // this is due to a bug found in oxide:  https://bugs.launchpad.net/oxide/+bug/1413964 and should be removed
+    // whenever asap
+    QString urlString = QString::null;
+    auto urlStringParts = _url.toString().split(DATA_URI_PREFIX);
+    if (urlStringParts.count() > 1) {
+        urlString = urlStringParts[1];
+    } else {
+        urlString = urlStringParts[0];
+    }
+
+    QByteArray data;
+    QMimeDatabase db;
+    QMimeType mimeType;  // init to an invalid type
+    QString extension = QString::null;
+    bool isBase64 = false;
+
+    foreach(const QMimeType& type, db.allMimeTypes()) {
+        // we need to check for each of the aliases that a mime type has
+        foreach(const QString& alias, type.aliases()) {
+            if (urlString.startsWith(alias)) {
+                DOWN_LOG(INFO) << "Data mime type is " << alias.toStdString();
+                mimeType = type;
+                urlString = urlString.replace(alias + ";", "");
+                break;
+            }
+        }
+
+        // we could have set the mime type and jumped outside the previous loop
+        if (mimeType.isValid()) {
+            break;
+        }
+    }
+    if (!mimeType.isValid()) {
+        LOG(INFO) << "MIME TYPE IS NOT VALID";
+        if (!urlString.startsWith(';')) {
+            DOWN_LOG(INFO) << "Database does not have type yet it is present.";
+            auto parts = urlString.split(';');
+            extension = parts[0].split('/')[1];
+            urlString = parts[1];
+        } else {
+            //defaults to text/plain;charset=US-ASCII.
+            mimeType = db.mimeTypeForData("text/plain");
+            urlString = urlString.split(';')[1];
+        }
+    }
+
+    if (urlString.startsWith("base64,")) {
+        DOWN_LOG(INFO) << "Data uri is base64 encoded";
+        isBase64 = true;
+        urlString = urlString.replace("base64,", "");
+    } else {
+        auto parts = urlString.split(',');
+        if (parts.count() != 2) {
+            DOWN_LOG(WARNING) << "Data uri malformed.";
+        } else {
+            urlString = parts[1];
+        }
+    }
+    // load the data from the string and write it in the file system, the uuid will be used as the
+    // file name + the mime type extension
+    if (isBase64) {
+        QByteArray encoded;
+        encoded.append(urlString);
+        data = QByteArray::fromBase64(encoded);
+    } else {
+        QByteArray encoded;
+        encoded.append(urlString);
+        data = QByteArray::fromPercentEncoding(encoded);
+    }
+
+    _totalSize = static_cast<qulonglong>(data.size());
+
+    // we need to update the file name using the mime type extension (the file name should be ok already), nevertheless
+    // we are going to write the data in the temp file so that the post processing of the download works correctky
+    QFileInfo fiFilePath(_filePath);
+    auto currentFileName = fiFilePath.fileName();
+    extension = (mimeType.isValid())?mimeType.suffixes()[0]:extension;
+    auto newPath = _filePath.replace(currentFileName, transferId() + "." + extension);
+
+    // unlock the old path and lock the new one
+    _fileNameMutex->unlockFileName(_filePath);
+    _filePath = _fileNameMutex->lockFileName(newPath);
+
+    DOWN_LOG(INFO) << "Data uri file path is '" << _filePath << "'";
+    _currentData->write(data);
+
+    // deal with the post processing of the created file
+    downloadPostProcessing(mimeType.aliases()[0]);
+}
+
 void
 FileDownload::onFinished() {
     TRACE << _url;
@@ -690,7 +771,6 @@ FileDownload::onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus) {
     if (exitCode == 0 && exitStatus == QProcess::NormalExit) {
         // remove the file since we are done with it
         cleanUpCurrentData();
-        emitFinished();
         // remove the file because that is the contract that we have with
         // the clients
         auto fileMan = FileManager::instance();
@@ -704,6 +784,7 @@ FileDownload::onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus) {
             LOG(INFO) << "Removing '" << _filePath << "'";
             fileMan->remove(_filePath);
         }
+        emitFinished();
     } else {
         auto standardOut = p->readAllStandardOutput();
         auto standardErr = p->readAllStandardError();
@@ -738,19 +819,41 @@ FileDownload::onOnlineStateChanged(bool online) {
     }
 }
 
+void
+FileDownload::onPropertiesChanged(const QVariantMap& changes) {
+    qDebug() << "Emit freedesktop.org changes";
+    auto signal = QDBusMessage::createSignal(
+        path(), PROPERTIES_INTERFACE, PROPERTIES_CHANGED);
+    signal << DOWNLOAD_INTERFACE;
+    signal << changes;
+
+    QStringList invalid;
+    signal << invalid;
+
+    DBusConnection::instance()->send(signal);
+}
+
 void
 FileDownload::init() {
     _requestFactory = RequestFactory::instance();
     _fileNameMutex = FileNameMutex::instance();
-    SystemNetworkInfo* networkInfo = SystemNetworkInfo::instance();
-    _connected = networkInfo->isOnline();
+    _connected = NetworkSession::instance()->isOnline();
     _downloading = false;
 
+    // applications that are confined are not allowed to set the click metadata.
+    if (isConfined() && _metadata.contains(Metadata::CLICK_PACKAGE_KEY)) {
+        _metadata.remove(Metadata::CLICK_PACKAGE_KEY);
+    }
+
     // connect to the network changed signals
-    CHECK(connect(networkInfo, &SystemNetworkInfo::onlineStateChanged,
+    CHECK(connect(NetworkSession::instance(), &NetworkSession::onlineStateChanged,
         this, &FileDownload::onOnlineStateChanged))
             << "Could not connect to signal";
 
+    CHECK(connect(this, &FileDownload::propertiesChanged,
+        this, &FileDownload::onPropertiesChanged))
+            << "Could not connect to signal";
+
     initFileNames();
 
     // ensure that the download is valid
@@ -758,6 +861,17 @@ FileDownload::init() {
         setIsValid(false);
         setLastError(QString("Invalid URL: '%1'").arg(_url.toString()));
     }
+
+    // ensure that if we are going to deflate the download that the hash is set
+    // to be empty. The reason for this is that if we deflate the hash wont be
+    // correctly checked
+    if (!_hash.isEmpty() && _metadata.contains(Metadata::DEFLATE_KEY)
+            && _metadata[Metadata::DEFLATE_KEY].toBool()) {
+        setIsValid(false);
+        setLastError(QString(
+            "Downloads that are set to be deflated cannot have a hash: '%1'").arg(
+                _hash));
+    }
 }
 
 void
@@ -803,6 +917,26 @@ FileDownload::flushFile() {
     return flushed;
 }
 
+bool
+FileDownload::hashIsValid() {
+    // if the hash is present we check it
+    if (!_hash.isEmpty()) {
+        emit processing(filePath());
+        _currentData->reset();
+        auto hashFactory = CryptographicHashFactory::instance();
+        QScopedPointer<CryptographicHash> hash(
+            hashFactory->createCryptographicHash(_algo, this));
+        // addData is smart enough to not load the entire file in memory
+        hash->addData(_currentData->device());
+        QString fileSig = QString(hash->result().toHex());
+        if (fileSig != _hash) {
+            DOWN_LOG(ERROR) << HASH_ERROR << fileSig << "!=" << _hash;
+            return false;
+        }
+    }
+    return true;
+}
+
 void
 FileDownload::initFileNames() {
     // the mutex will ensure that we do not have race conditions about
@@ -815,10 +949,8 @@ FileDownload::initFileNames() {
         _basename = UuidUtils::getDBusString(uuidFactory->createUuid());
     }
 
-    auto metadataMap = metadata();
-
-    if (!isConfined() && metadataMap.contains(Metadata::LOCAL_PATH_KEY)) {
-        _filePath = metadataMap[Metadata::LOCAL_PATH_KEY].toString();
+    if (!isConfined() && _metadata.contains(Metadata::LOCAL_PATH_KEY)) {
+        _filePath = _metadata[Metadata::LOCAL_PATH_KEY].toString();
         _tempFilePath = _fileNameMutex->lockFileName(
             _filePath + TEMP_EXTENSION);
 
@@ -837,6 +969,115 @@ FileDownload::initFileNames() {
     }
 }
 
+void
+FileDownload::downloadPostProcessing(const QString& contentType) {
+    TRACE << _url;
+
+    if(!hashIsValid()) {
+        emitError(HASH_ERROR);
+        return;
+    }
+
+    // there are three possible cases, in the first case we are requested
+    // to extract the file, in which case we start a special helper process.
+    // In the second case we are requested to execute a specific command, in
+    // either of these two cases we only raise the finished signal once the
+    // processing is complete. Or in the third case we have no special requests
+    // and we indicate that the download is finished.
+    if (_metadata.contains(Metadata::EXTRACT_KEY)
+            && _metadata[Metadata::EXTRACT_KEY].toBool()
+            && contentType == "application/zip") {
+
+        auto postDownloadProcess = ProcessFactory::instance()->createProcess();
+
+        CHECK(connect(postDownloadProcess, &Process::finished,
+                this, &FileDownload::onProcessFinished))
+                << "Could not connect to signal";
+        CHECK(connect(postDownloadProcess, &Process::error,
+                this, &FileDownload::onProcessError))
+                << "Could not connect to signal";
+
+        DOWN_LOG(INFO) << "Renaming '" << _tempFilePath << "'"
+                << "' to '" << _filePath << "'";
+
+        auto fileMan = FileManager::instance();
+        if (fileMan->exists(_tempFilePath)) {
+            LOG(INFO) << "Renaming: '" << _tempFilePath << "' to '"
+                    << _filePath << "'";
+            fileMan->rename(_tempFilePath, _filePath);
+        }
+
+        QFileInfo fileInfo(filePath());
+
+        QString command = HELPER_DIR + QString(QDir::separator()) + "udm-extractor";
+        QStringList args;
+        args << "--unzip";
+        args << "--path";
+        args << filePath();
+        args << "--destination";
+        args << fileInfo.dir().absolutePath();
+
+        DOWN_LOG(INFO) << "Executing" << command << args;
+        postDownloadProcess->start(command, args);
+        return;
+    } else if (_metadata.contains(Metadata::COMMAND_KEY)) {
+        // just emit processing if we DO NOT have a hash because else we
+        // already emitted it.
+        if (_hash.isEmpty()) {
+            emit processing(filePath());
+        }
+        // toStringList will return an empty list if it cannot be converted
+        QStringList commandData =
+                _metadata[Metadata::COMMAND_KEY].toStringList();
+        if (commandData.count() == 0) {
+            DOWN_LOG(ERROR) << "COMMAND DATA MISSING";
+            emitError(COMMAND_ERROR);
+            return;
+        } else {
+            // first item of the string list is the command
+            // the rest is the arguments
+            DOWN_LOG(INFO) << "Renaming '" << _tempFilePath << "'"
+                    << "' to '" << _filePath << "'";
+
+            auto fileMan = FileManager::instance();
+            if (fileMan->exists(_tempFilePath)) {
+                LOG(INFO) << "Renaming: '" << _tempFilePath << "' to '"
+                        << _filePath << "'";
+                fileMan->rename(_tempFilePath, _filePath);
+            }
+
+            QString command = commandData.at(0);
+            commandData.removeAt(0);
+            QStringList args;
+
+            foreach(const QString& arg, commandData) {
+                if (arg == Metadata::COMMAND_FILE_KEY)
+                    args << filePath();
+                else
+                    args << arg;
+            }
+
+            auto postDownloadProcess = ProcessFactory::instance()->createProcess();
+
+            // connect to signals so that we can tell the clients that
+            // the operation succeed
+
+            CHECK(connect(postDownloadProcess, &Process::finished,
+                    this, &FileDownload::onProcessFinished))
+                    << "Could not connect to signal";
+            CHECK(connect(postDownloadProcess, &Process::error,
+                    this, &FileDownload::onProcessError))
+                    << "Could not connect to signal";
+
+            DOWN_LOG(INFO) << "Executing" << command << args;
+            postDownloadProcess->start(command, args);
+            return;
+        }
+    } else {
+        emitFinished();
+    }
+}
+
 void
 FileDownload::emitFinished() {
     auto fileMan = FileManager::instance();
@@ -857,39 +1098,59 @@ FileDownload::emitFinished() {
     unlockFilePath();
 
     DOWN_LOG(INFO) << "EMIT finished" << filePath();
-    emit finished(_filePath);
+
+    if (_metadata.contains(Metadata::EXTRACT_KEY)
+        && _metadata[Metadata::EXTRACT_KEY].toBool()) {
+        // If we're finishing an extracted download then the returned file 
+        // path should be the directory we extracted the file into, as the
+        // download itself will have been cleaned up.
+        QFileInfo fileInfo(_filePath);
+        emit finished(fileInfo.dir().absolutePath());
+    } else {
+        emit finished(_filePath);
+    }
 }
 
 void
 FileDownload::unlockFilePath() {
-    if (!isConfined() && metadata().contains(Metadata::LOCAL_PATH_KEY)) {
+    if (!isConfined() && _metadata.contains(Metadata::LOCAL_PATH_KEY)) {
         _fileNameMutex->unlockFileName(_tempFilePath);
     } else {
         _fileNameMutex->unlockFileName(_filePath);
     }
 }
 
-QString
-FileDownload::filenameFromHTTPContentDisposition(const QString& value) {
-    auto keyValuePairs = value.split(';');
+void
+FileDownload::updateFileNamePerContentDisposition() {
+    // check if we have the content-type header, if we do we are going to change the
+    // file path that will be used by the download, do not do it if the app is
+    // unconfined
+    if ((_reply->hasRawHeader(CONTENT_DISPOSITION) && (
+            isConfined() || !_metadata.contains(Metadata::LOCAL_PATH_KEY)))) {
+        auto contentDisposition = _reply->rawHeader(CONTENT_DISPOSITION);
+        DOWN_LOG(INFO) << "Content-Disposition header" << contentDisposition;
 
-    foreach(const QString& valuePair, keyValuePairs) {
-        int valueStartPos = valuePair.indexOf('=');
+        if (contentDisposition.contains("filename")) {
+            auto serverName = HeaderParser::fileNameFromContentDisposition(
+                contentDisposition);
+            DOWN_LOG(INFO) << "Server name " << serverName;
 
-        if (valueStartPos < 0)
-            continue;
+            if (!serverName.isEmpty()) {
+                QFileInfo fiContentDisposition(serverName);
+                auto filename = fiContentDisposition.fileName();
+                // replace the filename of the current _filePath with the new one
+                QFileInfo fiFilePath(_filePath);
+                auto currentFileName = fiFilePath.fileName();
+                auto newPath = _filePath.replace(currentFileName, filename);
 
-        auto pair = valuePair.split('=');
-        if (pair.size() != 2 || pair[0].isEmpty() || pair[0].simplified() != "filename")
-            continue;
-
-        auto value = pair[1].replace("\"", "") // remove ""
-            .replace("'", "") // remove '
-            .simplified();  // remove white spaces
-        return value;
+                // unlock the old path and lock the new one
+                _fileNameMutex->unlockFileName(_filePath);
+                _filePath = _fileNameMutex->lockFileName(newPath);
+                DOWN_LOG(INFO) << "Content disposition based file path is '"
+                    << serverName << "'";
+            }
+        }
     }
-
-    return QString();
 }
 
 void
@@ -928,9 +1189,12 @@ FileDownload::buildRequest() {
             continue;
         request.setRawHeader(header.toUtf8(), data.toUtf8());
     }
-    // very important we must ensure that we do not decompress any download
-    // else we will have an error in the checksum for example #1224678
-    request.setRawHeader("Accept-Encoding", "identity");
+    if (!_metadata.contains(Metadata::DEFLATE_KEY) ||
+            !_metadata[Metadata::DEFLATE_KEY].toBool()) {
+        // very important we must ensure that we do not decompress any download
+        // else we will have an error in the checksum for example #1224678
+        request.setRawHeader("Accept-Encoding", "identity");
+    }
     return request;
 }
 
